@@ -28,11 +28,14 @@ import glob
 import logging
 import os
 import re
+import json
+import bleachbit
 from itertools import product
 
 # first party imports
-from bleachbit import Command, FileUtilities, General, Special, DeepScan
+from bleachbit import Command, FileUtilities, General, Special, DeepScan, Cookie as CookieMod  # mod=module
 from bleachbit import fs_scan_re_flags
+from bleachbit.Cookie import COOKIE_KEEP_LIST_FILENAME
 from bleachbit.Language import get_text as _
 
 if os.name == 'posix':
@@ -155,7 +158,7 @@ class FileActionProvider(ActionProvider):
                     # an environment variable, but each multi-value variable
                     # can have multiple values. They're a way to make CleanerML
                     # files more concise.
-                    _("Deep scan does not support multi-value variable."))
+                    _("Deep scan does not support multi-value variables."))
         if not any([self.object_type, self.regex, self.nregex,
                     self.wholeregex, self.nwholeregex]):
             # If the filter is not needed, bypass it for speed.
@@ -340,13 +343,15 @@ class AptAutoclean(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        # Checking executable allows auto-hide to work for non-APT systems
         assert os.name == 'posix'
-        if FileUtilities.exe_exists('apt-get'):
-            # pylint: disable=possibly-used-before-assignment
-            yield Command.Function(None,
-                                   Unix.apt_autoclean,
-                                   'apt-get autoclean')
+        # If apt-get is not installed, then enable fast auto-hide.
+        # The exe_exists() function is fast.
+        if not FileUtilities.exe_exists('apt-get'):
+            return
+        yield Command.Function(None,
+                               # pylint: disable=possibly-used-before-assignment
+                               Unix.apt_autoclean,
+                               'apt-get autoclean')
 
 
 class AptAutoremove(ActionProvider):
@@ -358,11 +363,11 @@ class AptAutoremove(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        # Checking executable allows auto-hide to work for non-APT systems
-        if FileUtilities.exe_exists('apt-get'):
-            yield Command.Function(None,
-                                   Unix.apt_autoremove,
-                                   'apt-get autoremove')
+        if not FileUtilities.exe_exists('apt-get'):
+            return
+        yield Command.Function(None,
+                               Unix.apt_autoremove,
+                               'apt-get autoremove')
 
 
 class AptClean(ActionProvider):
@@ -374,11 +379,11 @@ class AptClean(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        # Checking executable allows auto-hide to work for non-APT systems
-        if FileUtilities.exe_exists('apt-get'):
-            yield Command.Function(None,
-                                   Unix.apt_clean,
-                                   'apt-get clean')
+        if not FileUtilities.exe_exists('apt-get'):
+            return
+        yield Command.Function(None,
+                               Unix.apt_clean,
+                               'apt-get clean')
 
 
 class ChromeAutofill(FileActionProvider):
@@ -446,6 +451,89 @@ class ChromeKeywords(FileActionProvider):
                 _('Clean file'))
 
 
+class Cookie(FileActionProvider):
+
+    """Action to selectively clean cookies in Chromium/Mozilla browsers"""
+    action_key = 'cookie'
+
+    def get_commands(self):
+        keep_list = self._load_keep_list()
+
+        if not keep_list:
+            # If nothing is being kept, use regular delete for better performance
+            for path in self.get_paths():
+                yield Command.Delete(path)
+            return
+
+        # Otherwise, clean cookies using the keep list.
+        for path in self.get_paths():
+            def delete_func(p=path):
+                # perform deletion; return value is ignored by Command.Function for file paths
+                try:
+                    CookieMod.delete_cookies(p, keep_list, really_delete=True)
+                except Exception as e:
+                    logger.warning('Cookie cleaning failed on %s: %s', p, e)
+                return 0
+
+            def preview_func(p=path):
+                # return estimated file size reduction
+                try:
+                    result = CookieMod.delete_cookies(
+                        p, keep_list, really_delete=False)
+                    return result.get('file_size_reduction', 0)
+                except Exception as e:
+                    logger.warning(
+                        'Cookie cleaning preview failed on %s: %s', p, e)
+                    return 0
+
+            yield Command.Function(
+                path,
+                delete_func,
+                _('Clean cookies'),
+                preview_func)
+
+    def _load_keep_list(self):
+        """Load cookie domains to keep from options directory.
+
+        Supports either a list of strings (domains) or a list of objects
+        with a 'domain' key (cookie name field is ignored in v1).
+        """
+        path = os.path.join(bleachbit.options_dir, COOKIE_KEEP_LIST_FILENAME)
+        domains = set()
+        try:
+            with open(path, 'r', encoding='utf-8') as f:
+                data = json.load(f)
+            if isinstance(data, list):
+                for item in data:
+                    if isinstance(item, str):
+                        d = item
+                    elif isinstance(item, dict):
+                        d = item.get('domain')
+                    else:
+                        d = None
+                    if isinstance(d, str) and d:
+                        domains.add(d.lstrip('.').lower())
+        except (FileNotFoundError, json.JSONDecodeError, OSError):
+            pass
+        return domains
+
+    def _delete_cookies_with_keep_list(self, path):
+        """Delete cookies while honoring the keep list"""
+        keep_list = self._load_keep_list()
+        if not keep_list:
+            return 0
+        result = CookieMod.delete_cookies(path, keep_list, really_delete=True)
+        return result.get('file_size_reduction', 0)
+
+    def _preview_cookies_deletion(self, path):
+        """Preview cookies deletion, honoring the keep list"""
+        keep_list = self._load_keep_list()
+        if not keep_list:
+            return 0
+        result = CookieMod.delete_cookies(path, keep_list, really_delete=False)
+        return result.get('file_size_reduction', 0)
+
+
 class Delete(FileActionProvider):
 
     """Action to delete files"""
@@ -481,8 +569,10 @@ class Journald(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        if FileUtilities.exe_exists('journalctl'):
-            yield Command.Function(None, Unix.journald_clean, 'journalctl --vacuum-time=1')
+        # If journalctl is not installed, then enable fast auto-hide.
+        if not FileUtilities.exe_exists('journalctl'):
+            return
+        yield Command.Function(None, Unix.journald_clean, 'journalctl --vacuum-time=1')
 
 
 class Json(FileActionProvider):
@@ -642,7 +732,7 @@ class YumCleanAll(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        # Checking allows auto-hide to work for non-APT systems
+        # If yum is not installed, then enable fast auto-hide.
         if not FileUtilities.exe_exists('yum'):
             return
 
@@ -661,7 +751,7 @@ class DnfCleanAll(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        # Checking allows auto-hide to work for non-APT systems
+        # If dnf is not installed, then enable fast auto-hide.
         if not FileUtilities.exe_exists('dnf'):
             return
 
@@ -680,7 +770,7 @@ class DnfAutoremove(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
-        # Checking allows auto-hide to work for non-APT systems
+        # If dnf is not installed, then enable fast auto-hide.
         if not FileUtilities.exe_exists('dnf'):
             return
 
@@ -699,6 +789,9 @@ class PacmanCache(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
+        if not FileUtilities.exe_exists('paccache'):
+            return
+
         yield Command.Function(
             None,
             Unix.pacman_cache,
@@ -714,8 +807,11 @@ class SnapDisabled(ActionProvider):
         ActionProvider.__init__(self, action_element, path_vars)
 
     def get_commands(self):
+        # If snap is not installed, then enable fast auto-hide.
+        if not FileUtilities.exe_exists('snap'):
+            return
         yield Command.Function(
             None,
             Unix.snap_disabled_clean,
             'snap remove disabled',
-            Unix.snap_disabled_preview)
+            preview_func=Unix.snap_disabled_preview)
