@@ -1,21 +1,8 @@
-# vim: ts=4:sw=4:expandtab
-
-# BleachBit
-# Copyright (C) 2008-2025 Andrew Ziem
-# https://www.bleachbit.org
+# SPDX-License-Identifier: GPL-3.0-or-later
+# Copyright (c) 2008-2026 Andrew Ziem.
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This work is licensed under the terms of the GNU GPL, version 3 or
+# later.  See the COPYING file in the top-level directory.
 
 """
 Command line interface
@@ -107,14 +94,37 @@ def args_to_operations_list(preset, all_but_warning):
     return args
 
 
-def args_to_operations(args, preset, all_but_warning):
-    """Read arguments and return list of operations as dictionary"""
+def args_to_operations(args, preset, all_but_warning, excludes=None):
+    """Convert command-line arguments to a dictionary of operations
+
+    Args:
+        args: List of cleaner.option strings (e.g., ['system.tmp', 'firefox.cache'])
+        preset: Boolean indicating whether to use saved preset operations
+        all_but_warning: Boolean indicating whether to include all non-warning operations
+        excludes: Optional list of cleaner.option strings to remove from operations
+
+    Returns:
+        Dictionary mapping cleaner IDs to lists of option IDs
+        Example: {'system': ['tmp'], 'firefox': ['cache']}
+    """
     list(register_cleaners())
     operations = {}
     if not args:
         args = []
-    args = set(args + args_to_operations_list(preset, all_but_warning))
-    for arg in args:
+
+    if excludes is None:
+        excludes = []
+    positive_args = set(
+        args + args_to_operations_list(preset, all_but_warning))
+
+    def fix_deprecated(cid, oid):
+        if 'system' == cid and 'free_disk_space' == oid:
+            logger.info(
+                "Change 'system.free_disk_space' (deprecated) to 'system.empty_space'")
+            return 'empty_space'
+        return oid
+
+    for arg in positive_args:
         if 2 != len(arg.split('.')):
             logger.warning(_("not a valid cleaner: %s"), arg)
             continue
@@ -128,25 +138,57 @@ def args_to_operations(args, preset, all_but_warning):
                 for (option_id2, _o_name) in backends[cleaner_id].get_options()
             ]
             continue
+        # backwards compatibility
+        option_id = fix_deprecated(cleaner_id, option_id)
         # add the specified option
         if cleaner_id not in operations:
+            # initialize list of options for this cleaner
             operations[cleaner_id] = []
         if option_id not in operations[cleaner_id]:
+            # add option to list
             operations[cleaner_id].append(option_id)
+
+    for arg in excludes:
+        if '*' in arg or '?' in arg:
+            logger.error(
+                _("Wildcard characters are not allowed in --except: %s"), arg)
+            # Exit to avoid over-cleaning.
+            sys.exit(1)
+        if 2 != len(arg.split('.')):
+            logger.error(_("not a valid cleaner: %s"), arg)
+            # Exit to avoid over-cleaning.
+            sys.exit(1)
+        (cleaner_id, option_id) = arg.split('.')
+        option_id = fix_deprecated(cleaner_id, option_id)
+        if cleaner_id in operations and option_id in operations[cleaner_id]:
+            operations[cleaner_id].remove(option_id)
+            if not operations[cleaner_id]:
+                del operations[cleaner_id]
+
     for (k, v) in operations.items():
         operations[k] = sorted(v)
     return operations
 
 
-def process_cmd_line():
-    """Parse the command line and execute given commands."""
+def _split_excludes(values):
+    excludes = []
+    for value in values:
+        if value is None:
+            continue
+        excludes.extend(item.strip()
+                        for item in value.split(',') if item.strip())
+    return excludes
+
+
+def parse_cmd_line(argv=None):
+    """Parse the command line and return parser, options, args, and excludes."""
 
     # TRANSLATORS: This is the command line usage.  Don't translate
     # %prog, but do translate options, cleaner, and option.
     # Don't translate and add "usage:" - it gets added by Python.
     # More information about the command line is here
     # https://www.bleachbit.org/documentation/command-line
-    usage = _("usage: %prog [options] cleaner.option1 cleaner.option2")
+    usage = _("usage: %prog [options] cleaner.option1 [cleaner.option2 ...]")
     parser = optparse.OptionParser(usage)
     parser.add_option("-l", "--list-cleaners", action="store_true",
                       help=_("list cleaners"))
@@ -159,8 +201,9 @@ def process_cmd_line():
                       help=_("run cleaners to delete files and make other permanent changes"))
     parser.add_option("-s", "--shred", action="store_true",
                       help=_("shred specific files or folders"))
-    parser.add_option("-w", "--wipe-free-space", action="store_true",
-                      help=_("wipe free space in the given paths"))
+    parser.add_option("-w", "--wipe-empty-space", "--wipe-free-space",
+                      action="store_true", dest="wipe_empty_space",
+                      help=_("wipe empty space in the given paths"))
     parser.add_option('-o', '--overwrite', action='store_true',
                       help=_('overwrite files to hide contents'))
     parser.add_option("--gui", action="store_true",
@@ -169,6 +212,8 @@ def process_cmd_line():
                       help=_("use options set in the graphical interface"))
     parser.add_option("--all-but-warning", action="store_true",
                       help=_("enable all options that do not have a warning"))
+    parser.add_option("--except", dest="excludes", action="append", default=[],
+                      help=_("exclude cleaner options (can be repeated, comma-separated)"))
     parser.add_option(
         '--debug', help=_("set log level to verbose"), action="store_true")
     parser.add_option('--debug-log', help=_("log debug messages to file"))
@@ -176,15 +221,26 @@ def process_cmd_line():
                       help=_("show system information"))
     parser.add_option("-v", "--version", action="store_true",
                       help=_("output version information and exit"))
+    parser.add_option('--pot', action='store_true',
+                      help=optparse.SUPPRESS_HELP)
+    parser.add_option("--no-delete-confirmation", action="store_false",
+                      dest="delete_confirmation",
+                      help=optparse.SUPPRESS_HELP)
+    parser.add_option("--no-load-cleaners", action="store_false",
+                      dest="load_cleaners",
+                      help=optparse.SUPPRESS_HELP)
+    parser.add_option("--no-check-online-updates", action="store_false",
+                      dest="check_online_updates",
+                      help=optparse.SUPPRESS_HELP)
+    parser.add_option("--no-first-start", action="store_false",
+                      dest="first_start",
+                      help=optparse.SUPPRESS_HELP)
 
     if 'nt' == os.name:
         uac_help = _("do not prompt for administrator privileges")
-    else:
-        uac_help = optparse.SUPPRESS_HELP
-    parser.add_option("--no-uac", action="store_true", help=uac_help)
-    parser.add_option('--pot', action='store_true',
-                      help=optparse.SUPPRESS_HELP)
-    if 'nt' == os.name:
+        parser.add_option("--no-uac", action="store_true", help=uac_help)
+
+        parser.add_option('--uac-sid-token', help=optparse.SUPPRESS_HELP)
         parser.add_option("--update-winapp2", action="store_true",
                           help=_("update winapp2.ini, if a new version is available"))
 
@@ -198,23 +254,43 @@ def process_cmd_line():
     def expand_context_menu_option(_option, _opt, _value, parser):
         setattr(parser.values, 'gui', True)
         setattr(parser.values, 'exit', True)
+        setattr(parser.values, 'load_cleaners', False)
+        setattr(parser.values, 'check_online_updates', False)
+        setattr(parser.values, 'first_start', False)
     parser.add_option("--context-menu", action="callback", callback=expand_context_menu_option,
                       help=optparse.SUPPRESS_HELP)
 
-    (options, args) = parser.parse_args()
+    (options, args) = parser.parse_args(args=argv)
+    excludes = _split_excludes(options.excludes)
+    return parser, options, args, excludes
 
-    cmd_list = (options.list_cleaners, options.wipe_free_space,
-                options.preview, options.clean)
+
+def process_cmd_line():
+    """Parse the command line and execute given commands."""
+
+    parser, options, args, excludes = parse_cmd_line()
+
+    for opt in ('delete_confirmation', 'load_cleaners',
+                'check_online_updates', 'first_start'):
+        if hasattr(options, opt) and getattr(options, opt) is not None:
+            Options.options.set_override(opt, getattr(options, opt))
+
+    cmd_list = (options.list_cleaners,
+                options.clean,
+                options.preview,
+                options.shred,
+                options.wipe_empty_space)
     cmd_count = sum(x is True for x in cmd_list)
     if cmd_count > 1:
         logger.error(
-            _('Specify only one of these commands: --list-cleaners, --wipe-free-space, --preview, --clean'))
+            _('Specify only one of these commands: --list-cleaners, --wipe-empty-space, --preview, --clean, --shred'))
         sys.exit(1)
 
     did_something = False
     if options.debug:
-        # set in __init__ so it takes effect earlier
-        pass
+        # Debug is set in __init__ so it takes effect earlier.
+        # Also, set the override.
+        Options.options.set_override('debug', True)
     elif options.preset:
         # but if --preset is given, check if GUI option sets debug
         if Options.options.get('debug'):
@@ -227,7 +303,7 @@ def process_cmd_line():
     if options.version:
         print("""
 BleachBit version %s
-Copyright (C) 2008-2025 Andrew Ziem.  All rights reserved.
+Copyright (C) 2008-2026 Andrew Ziem.  All rights reserved.
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
@@ -247,38 +323,40 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
         from bleachbit.CleanerML import create_pot
         create_pot()
         sys.exit(0)
-    if options.wipe_free_space:
+    if options.wipe_empty_space:
         if len(args) < 1:
-            logger.error(_("No directories given for --wipe-free-space"))
+            logger.error(_("No directories given for --wipe-empty-space"))
             sys.exit(1)
-        logger.info(_("Wiping free space can take a long time."))
+        logger.info(_("Wiping empty space can take a long time."))
         for wipe_path in args:
-            logger.info('Wiping free space in path: %s', wipe_path)
+            logger.info(_("Wipe empty space in %s"), wipe_path)
             import bleachbit.FileUtilities
             for _ret in bleachbit.FileUtilities.wipe_path(wipe_path):
                 pass
         sys.exit(0)
     if options.preview or options.clean:
         operations = args_to_operations(
-            args, options.preset, options.all_but_warning)
+            args, options.preset, options.all_but_warning, excludes)
         if not operations:
-            logger.error(_("No work to do. Specify options."))
+            logger.error(_("No operations selected. Specify cleaner options."))
             sys.exit(1)
     if options.overwrite:
         if not options.clean or options.shred:
             logger.warning(
                 _("--overwrite is intended only for use with --clean"))
-        Options.options.set('shred', True, commit=False)
+        Options.options.set_override('shred', True)
     if options.clean or options.preview:
         preview_or_clean(operations, options.clean)
         sys.exit(0)
     if options.gui:
-        import bleachbit.GUI
-        app = bleachbit.GUI.Bleachbit(
-            uac=not options.no_uac, shred_paths=args, auto_exit=options.exit)
+        import bleachbit.GuiApplication
+        enable_uac = os.name == 'nt' and not options.no_uac
+        app = bleachbit.GuiApplication.Bleachbit(
+            uac=enable_uac, shred_paths=args, auto_exit=options.exit)
         sys.exit(app.run())
     if options.shred:
         # delete arbitrary files without GUI
+        Options.options.set_override('first_start', False)
         # create a temporary cleaner object
         backends['_gui'] = create_simple_cleaner(args)
         operations = {'_gui': ['files']}
