@@ -1,21 +1,8 @@
-# vim: ts=4:sw=4:expandtab
-
-# BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
-# https://www.bleachbit.org
+# SPDX-License-Identifier: GPL-3.0-or-later
+# Copyright (c) 2008-2026 Andrew Ziem.
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This work is licensed under the terms of the GNU GPL, version 3 or
+# later.  See the COPYING file in the top-level directory.
 
 """
 Perform (or assist with) cleaning operations.
@@ -26,30 +13,21 @@ import logging
 import os.path
 import re
 import sys
+import tempfile
 
-from bleachbit import _
+from bleachbit.Language import get_text as _
 from bleachbit.FileUtilities import children_in_directory
 from bleachbit.Options import options
-from bleachbit import Command, FileUtilities, Memory, Special
-
-
-# Suppress GTK warning messages while running in CLI #34
-import warnings
-warnings.simplefilter("ignore", Warning)
-try:
-    from bleachbit.GuiBasic import Gtk, Gdk
-    HAVE_GTK = Gdk.get_default_root_window() is not None
-except (ImportError, RuntimeError, ValueError):
-    # ImportError happens when GTK is not installed.
-    # RuntimeError can happen when X is not available (e.g., cron, ssh).
-    # ValueError seen on BleachBit 3.0 with GTK 3 (GitHub issue 685)
-    HAVE_GTK = False
-
+from bleachbit import Action, CleanerML, Command, FileUtilities, Memory, Special
+from bleachbit.GtkShim import Gtk, Gdk, HAVE_GTK
+from bleachbit.Wipe import wipe_path
 
 if 'posix' == os.name:
     from bleachbit import Unix
 elif 'nt' == os.name:
     from bleachbit import Windows
+elif os.name not in ('posix', 'nt'):
+    raise RuntimeError(f"Unknown OS '{os.name}'")
 
 
 # a module-level variable for holding cleaners
@@ -80,9 +58,9 @@ class Cleaner:
         """Register option (such as 'cache')"""
         self.options[option_id] = (name, description)
 
-    def add_running(self, detection_type, pathname):
+    def add_running(self, detection_type, pathname, same_user=False):
         """Add a way to detect this program is currently running"""
-        self.running.append((detection_type, pathname))
+        self.running.append((detection_type, pathname, same_user))
 
     def auto_hide(self):
         """Return boolean whether it is OK to automatically hide this
@@ -106,7 +84,7 @@ class Cleaner:
             if option_id == action[0]:
                 yield from action[1].get_commands()
         if option_id not in self.options:
-            raise RuntimeError("Unknown option '%s'" % option_id)
+            raise RuntimeError(f"Unknown option '{option_id}'")
 
     def get_deep_scan(self, option_id):
         """Get dictionary used to build a deep scan"""
@@ -117,7 +95,7 @@ class Cleaner:
                 except StopIteration:
                     return
         if option_id not in self.options:
-            raise RuntimeError("Unknown option '%s'" % option_id)
+            raise RuntimeError(f"Unknown option '{option_id}'")
 
     def get_description(self):
         """Brief description of the cleaner"""
@@ -149,15 +127,12 @@ class Cleaner:
             return self.warnings[option_id]
         return None
 
-    def is_running(self):
-        """Return whether the program is currently running"""
+    def is_process_running(self):
+        """Return whether the process is currently running"""
         logger = logging.getLogger(__name__)
-        for running in self.running:
-            test = running[0]
-            pathname = running[1]
+        for (test, pathname, same_user) in self.running:
             if 'exe' == test:
-                if ('posix' == os.name and Unix.is_running(pathname)) or \
-                   ('nt' == os.name and Windows.is_process_running(pathname)):
+                if _is_process_running(pathname, same_user):
                     logger.debug("process '%s' is running", pathname)
                     return True
             elif 'pathname' == test:
@@ -168,8 +143,7 @@ class Cleaner:
                             "file '%s' exists indicating '%s' is running", globbed, self.name)
                         return True
             else:
-                raise RuntimeError(
-                    "Unknown running-detection test '%s'" % test)
+                raise RuntimeError(f"Unknown running-detection test '{test}'")
         return False
 
     def is_usable(self):
@@ -197,9 +171,12 @@ class OpenOfficeOrg(Cleaner):
 
         # reference: http://katana.oooninja.com/w/editions_of_openoffice.org
         if 'posix' == os.name:
-            self.prefixes = ["~/.ooo-2.0", "~/.openoffice.org2",
-                             "~/.openoffice.org2.0", "~/.openoffice.org/3",
-                             "~/.ooo-dev3"]
+            self.prefixes = ["~/.ooo-2.0",
+                             "~/.openoffice.org2",
+                             "~/.openoffice.org2.0",
+                             "~/.openoffice.org/3",
+                             "~/.ooo-dev3",
+                             "~/.config/libreoffice/4"]
         if 'nt' == os.name:
             self.prefixes = [
                 "$APPDATA\\OpenOffice.org\\3", "$APPDATA\\OpenOffice.org2"]
@@ -237,7 +214,8 @@ class OpenOfficeOrg(Cleaner):
 
         if 'recent_documents' == option_id:
             for prefix in self.prefixes:
-                for path in FileUtilities.expand_glob_join(prefix, "user/registry/data/org/openoffice/Office/Common.xcu"):
+                for path in FileUtilities.expand_glob_join(prefix,
+                                                           "user/registry/data/org/openoffice/Office/Common.xcu"):
                     if os.path.lexists(path):
                         yield Command.Function(path,
                                                Special.delete_ooo_history,
@@ -246,7 +224,8 @@ class OpenOfficeOrg(Cleaner):
                 #       Apache OpenOffice.org 3.4.1 from openoffice.org on Ubuntu 13.04
                 # %AppData%\OpenOffice.org\3\user\registrymodifications.xcu
                 # Apache OpenOffice.org 3.4.1 from openoffice.org on Windows XP
-                for path in FileUtilities.expand_glob_join(prefix, "user/registrymodifications.xcu"):
+                for path in FileUtilities.expand_glob_join(prefix,
+                                                           "user/registrymodifications.xcu"):
                     if os.path.lexists(path):
                         yield Command.Function(path,
                                                Special.delete_office_registrymodifications,
@@ -293,7 +272,7 @@ class System(Cleaner):
         #
         # options just for Linux
         #
-        if sys.platform.startswith('linux'):
+        if sys.platform == 'linux':
             self.add_option('memory', _('Memory'),
                             # TRANSLATORS: 'free' means 'unallocated'
                             _('Wipe the swap and free memory'))
@@ -316,6 +295,7 @@ class System(Cleaner):
             # the uninstallers for software updates.
             self.add_option('updates', _('Update uninstallers'), _(
                 'Delete uninstallers for Microsoft updates including hotfixes, service packs, and Internet Explorer updates'))
+            self.set_warning('updates', _('This option may prevent uninstalling some updates.'))
 
         #
         # options for GTK+
@@ -332,11 +312,11 @@ class System(Cleaner):
         # files and folders will be erased.
         self.add_option('custom', _('Custom'), _(
             'Delete user-specified files and folders'))
-        # TRANSLATORS: 'free' means 'unallocated'
-        self.add_option('free_disk_space', _('Free disk space'),
-                        # TRANSLATORS: 'free' means 'unallocated'
-                        _('Overwrite free disk space to hide deleted files'))
-        self.set_warning('free_disk_space', _('This option is very slow.'))
+        # TRANSLATORS: 'empty' means 'unallocated'
+        self.add_option('empty_space', _('Empty space'),
+                        # TRANSLATORS: 'empty' means 'unallocated'
+                        _('Wipe empty space to hide deleted files'))
+        self.set_warning('empty_space', _('Wiping empty space removes traces of files that were deleted without shredding, but it will not free additional disk space. The process can take a very long time and may temporarily slow your computer. It is not necessary if your drive is protected with full-disk encryption. The method works best on traditional hard drives. On solid-state drives, it is less reliable, and frequent use contributes to wear.'))
         self.add_option(
             'tmp', _('Temporary files'), _('Delete the temporary files'))
 
@@ -356,14 +336,16 @@ class System(Cleaner):
         if 'custom' == option_id:
             for (c_type, c_path) in options.get_custom_paths():
                 if 'file' == c_type:
-                    yield Command.Delete(c_path)
+                    if os.path.lexists(c_path):
+                        yield Command.Delete(c_path)
                 elif 'folder' == c_type:
-                    for path in children_in_directory(c_path, True):
-                        yield Command.Delete(path)
-                    yield Command.Delete(c_path)
+                    if os.path.lexists(c_path):
+                        for path in children_in_directory(c_path, True):
+                            yield Command.Delete(path)
+                        yield Command.Delete(c_path)
                 else:
                     raise RuntimeError(
-                        'custom folder has invalid type %s' % c_type)
+                        f'custom folder has invalid type {c_type}')
 
         # menu
         menu_dirs = ['~/.local/share/applications',
@@ -381,6 +363,7 @@ class System(Cleaner):
             for path in menu_dirs:
                 dirname = os.path.expanduser(path)
                 for filename in children_in_directory(dirname, False):
+                    # pylint: disable=possibly-used-before-assignment
                     if filename.endswith('.desktop') and Unix.is_broken_xdg_desktop(filename):
                         yield Command.Delete(filename)
 
@@ -438,7 +421,7 @@ class System(Cleaner):
                     yield Command.Delete(globbed)
 
         # memory
-        if sys.platform.startswith('linux') and 'memory' == option_id:
+        if sys.platform == 'linux' and 'memory' == option_id:
             yield Command.Function(None, Memory.wipe_memory, _('Memory'))
 
         # memory dump
@@ -472,9 +455,9 @@ class System(Cleaner):
                 yield 0
 
             xbel_pathnames = [
-                    '~/.recently-used.xbel',
-                    '~/.local/share/recently-used.xbel*',
-                    '~/snap/*/*/.local/share/recently-used.xbel']
+                '~/.recently-used.xbel',
+                '~/.local/share/recently-used.xbel*',
+                '~/snap/*/*/.local/share/recently-used.xbel']
             for path1 in xbel_pathnames:
                 for path2 in glob.iglob(os.path.expanduser(path1)):
                     if os.path.lexists(path2):
@@ -502,7 +485,8 @@ class System(Cleaner):
 
         # temporary files
         if 'nt' == os.name and 'tmp' == option_id:
-            dirnames = [os.path.expandvars(r'%temp%'), os.path.expandvars("%windir%\\temp\\")]
+            dirnames = [os.path.expandvars(
+                r'%temp%'), os.path.expandvars("%windir%\\temp\\")]
             # whitelist the folder %TEMP%\Low but not its contents
             # https://bugs.launchpad.net/bleachbit/+bug/1421726
             for dirname in dirnames:
@@ -543,18 +527,18 @@ class System(Cleaner):
                 return 0
             yield Command.Function(None, clear_clipboard, _('Clipboard'))
 
-        # overwrite free space
+        # wipe empty space
         shred_drives = options.get_list('shred_drives')
-        if 'free_disk_space' == option_id and shred_drives:
+        if 'empty_space' == option_id and shred_drives:
             for pathname in shred_drives:
-                # TRANSLATORS: 'Free' means 'unallocated.'
+                # TRANSLATORS: 'Empty' means 'unallocated.'
                 # %s expands to a path such as C:\ or /tmp/
-                display = _("Overwrite free disk space %s") % pathname
+                display = _("Wipe empty space in %s") % pathname
 
-                def wipe_path_func():
+                def wipe_path_func(path=pathname):
                     # Yield control to GTK idle because this process
                     # is very slow.  Also display progress.
-                    yield from FileUtilities.wipe_path(pathname, idle=True)
+                    yield from wipe_path(path, idle=True)
                     yield 0
                 yield Command.Function(None, wipe_path_func, display)
 
@@ -575,6 +559,7 @@ class System(Cleaner):
         if 'nt' == os.name and 'recycle_bin' == option_id:
             # This method allows shredding
             recycled_any = False
+            # pylint: disable=possibly-used-before-assignment
             for path in Windows.get_recycle_bin():
                 recycled_any = True
                 yield Command.Delete(path)
@@ -584,12 +569,11 @@ class System(Cleaner):
 
             # This is a hack to refresh the icon.
             def empty_recycle_bin_func():
-                import tempfile
                 tmpdir = tempfile.mkdtemp()
                 Windows.move_to_recycle_bin(tmpdir)
                 try:
                     Windows.empty_recycle_bin(None, True)
-                except:
+                except Exception:
                     logging.getLogger(__name__).info(
                         'error in empty_recycle_bin()', exc_info=True)
                 yield 0
@@ -635,7 +619,24 @@ class System(Cleaner):
             '^' + os.path.expanduser('~/.cache/ibus/'),
             # Linux Bluetooth daemon obexd directory is typically empty, so be careful
             # not to delete the empty directory.
-            '^' + os.path.expanduser('~/.cache/obexd($|/)')]
+            '^' + os.path.expanduser('~/.cache/obexd($|/)'),
+            # KDE/Plasma cache files
+            # https://github.com/bleachbit/bleachbit/issues/1853
+            '^' + os.path.expanduser('~/.cache/kwin($|/)'),  # folder
+            # folder
+            '^' + os.path.expanduser('~/.cache/mesa_shader_cache($|/)'),
+            '^' + os.path.expanduser('~/.cache/plasmashell($|/)'),  # folder
+            '^' + os.path.expanduser('~/.cache/icon-cache.kcache$'),  # file
+            # file
+            r'^' + os.path.expanduser(r'~/.cache/plasma_theme_.*\.kcache$'),
+            '^' + os.path.expanduser('~/.cache/drkonqi($|/)'),  # folder
+            # folder
+            '^' + os.path.expanduser('~/.cache/mesa_shader_cache_db($|/)'),
+            # folder
+            '^' + os.path.expanduser('~/.cache/qtshadercache-[^/]+($|/)'),
+            # file
+            '^' + os.path.expanduser('~/.cache/plasma_theme_default.kcache$')]
+
         for regex in regexes:
             self.regexes_compiled.append(re.compile(regex))
 
@@ -652,8 +653,17 @@ class System(Cleaner):
         return False
 
 
+def _is_process_running(exename, require_same_user):
+    if 'posix' == os.name:
+        return Unix.is_process_running(exename, require_same_user)
+    elif 'nt' == os.name:
+        return Windows.is_process_running(exename, require_same_user)
+    raise NotImplementedError('_is_process_running: Unsupported platform')
+
+
 def register_cleaners(cb_progress=lambda x: None, cb_done=lambda: None):
     """Register all known cleaners: system, CleanerML, and Winapp2"""
+    # pylint: disable=global-variable-not-assigned
     global backends
 
     # wipe out any registrations
@@ -664,14 +674,19 @@ def register_cleaners(cb_progress=lambda x: None, cb_done=lambda: None):
     backends["openofficeorg"] = OpenOfficeOrg()
     backends["system"] = System()
 
+    if not options.get("load_cleaners"):
+        cb_done()
+        yield False
+        return
+
     # register CleanerML cleaners
-    from bleachbit import CleanerML
     cb_progress(_('Loading native cleaners.'))
     yield from CleanerML.load_cleaners(cb_progress)
 
     # register Winapp2.ini cleaners
     if 'nt' == os.name:
         cb_progress(_('Importing cleaners from Winapp2.ini.'))
+        # pylint: disable=import-outside-toplevel
         from bleachbit import Winapp
         yield from Winapp.load_cleaners(cb_progress)
 
@@ -686,16 +701,15 @@ def create_simple_cleaner(paths):
     cleaner.add_option(option_id='files', name='', description='')
     cleaner.name = _("System")  # shows up in progress bar
 
-    from bleachbit import Action
-
     class CustomFileAction(Action.ActionProvider):
+        """Custom file action"""
         action_key = '__customfileaction'
 
         def get_commands(self):
             for path in paths:
                 if not isinstance(path, (str)):
                     raise RuntimeError(
-                        'expected path as string but got %s' % str(path))
+                        f'expected path as string but got {str(path)}')
                 if not os.path.isabs(path):
                     path = os.path.abspath(path)
                 if os.path.isdir(path):
@@ -707,27 +721,25 @@ def create_simple_cleaner(paths):
     return cleaner
 
 
-def create_wipe_cleaner(path):
-    """Wipe free disk space of arbitrary paths (used in GUI)"""
+def create_wipe_empty_space_cleaner(path):
+    """Wipe empty space of arbitrary paths (used in GUI)"""
     cleaner = Cleaner()
     cleaner.add_option(
-        option_id='free_disk_space', name='', description='')
+        option_id='empty_space', name='', description='')
     cleaner.name = ''
 
     # create a temporary cleaner object
-    display = _("Overwrite free disk space %s") % path
+    display = _("Wipe empty space %s") % path
 
     def wipe_path_func():
-        yield from FileUtilities.wipe_path(path, idle=True)
+        yield from bleachbit.Wipe.wipe_path(path, idle=True)
         yield 0
 
-    from bleachbit import Action
-
     class CustomWipeAction(Action.ActionProvider):
         action_key = '__customwipeaction'
 
         def get_commands(self):
             yield Command.Function(None, wipe_path_func, display)
     provider = CustomWipeAction(None)
-    cleaner.add_action('free_disk_space', provider)
+    cleaner.add_action('empty_space', provider)
     return cleaner
