@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -21,24 +21,19 @@
 Basic GUI code
 """
 
-from bleachbit import _, ModuleNotFoundError
-
+# standard library
 import os
 
-try:
-    import gi
-except ModuleNotFoundError as e:
-    print('*'*60)
-    print('Please install PyGObject')
-    print('https://pygobject.readthedocs.io/en/latest/getting_started.html')
-    print('*'*60)
-    raise e
-gi.require_version('Gtk', '3.0')
-from gi.repository import Gtk, Gdk  # keep after gi.require_version()
-
+# local import
+from bleachbit.GtkShim import Gtk, Gdk, GLib, require_gtk
+from bleachbit.Language import get_text as _
+from bleachbit.Options import options
 if os.name == 'nt':
     from bleachbit import Windows
 
+# Ensure GTK is available for this GUI module
+require_gtk()
+
 
 def browse_folder(parent, title, multiple, stock_button):
     """Ask the user to select a folder.  Return the full path or None."""
@@ -144,6 +139,11 @@ def delete_confirmation_dialog(parent, mention_preview, shred_settings=False):
     question.set_line_wrap(True)
     vbox.pack_start(question, False, True, 0)
 
+    if options.get('expert_mode'):
+        cb_popup = Gtk.CheckButton(label=_("Confirm before delete"))
+        cb_popup.set_active(options.get('delete_confirmation'))
+        vbox.pack_start(cb_popup, False, True, 0)
+
     dialog.get_content_area().pack_start(vbox, False, True, 0)
     dialog.get_content_area().set_spacing(10)
 
@@ -153,10 +153,59 @@ def delete_confirmation_dialog(parent, mention_preview, shred_settings=False):
 
     dialog.show_all()
     ret = dialog.run()
+    if options.get('expert_mode'):
+        options.set('delete_confirmation', cb_popup.get_active())
     dialog.destroy()
     return ret == Gtk.ResponseType.ACCEPT
 
 
+def warning_confirm_dialog(parent, option_name, warning_text, show_checkbox=True):
+    """Show a warning dialog when enabling an option.
+
+    Returns tuple (confirmed: bool, remember_choice: bool).
+    """
+    dialog = Gtk.Dialog(title=_('Enable %(option)s') % {'option': option_name},
+                        transient_for=parent,
+                        modal=True,
+                        destroy_with_parent=True)
+    content = dialog.get_content_area()
+    content.set_border_width(12)
+    content.set_spacing(10)
+
+    warning_label = Gtk.Label(label=warning_text)
+    warning_label.set_line_wrap(True)
+    warning_label.set_xalign(0.0)
+    content.pack_start(warning_label, False, False, 0)
+
+    remember_cb = Gtk.CheckButton(
+        label=_('Remember my choice for %(option)s') % {'option': option_name})
+    remember_cb.set_halign(Gtk.Align.START)
+    if show_checkbox:
+        content.pack_start(remember_cb, False, False, 0)
+
+    cancel_button = dialog.add_button(_('_Cancel'), Gtk.ResponseType.CANCEL)
+    enable_button = dialog.add_button(_("_Enable anyway"), Gtk.ResponseType.OK)
+    enable_button.get_style_context().add_class('destructive-action')
+    dialog.set_default_response(Gtk.ResponseType.CANCEL)
+    cancel_button.grab_focus()
+
+    # Disable enable button for a moment to prevent accident by double-clicking.
+    enable_button.set_sensitive(False)
+
+    def enable_button_after_delay():
+        enable_button.set_sensitive(True)
+        return False
+
+    GLib.timeout_add(500, enable_button_after_delay)
+
+    dialog.show_all()
+    response = dialog.run()
+    remember_choice = response == Gtk.ResponseType.OK and show_checkbox and remember_cb.get_active()
+    dialog.destroy()
+
+    return response == Gtk.ResponseType.OK, remember_choice
+
+
 def message_dialog(parent, msg, mtype=Gtk.MessageType.ERROR, buttons=Gtk.ButtonsType.OK, title=None):
     """Convenience wrapper for Gtk.MessageDialog"""
 
