@@ -38,6 +38,8 @@ class CommonTestCase(common.BleachbitTestCase):
             'C.utf8',
             'C',
             'de_DE.iso88591',
+            'de-CH', # seen in Fedora in Docker
+            'en-US',
             'en_US',
             'en',
             'fr_FR.utf8',
@@ -49,7 +51,7 @@ class CommonTestCase(common.BleachbitTestCase):
             'zh_Hant',
         ]
 
-        invalid_codes = ['e', 'en_', 'english', 'en_US_', '123', 'en-US',
+        invalid_codes = ['e', 'en_', 'english', 'en_US_', '123',
                          'en_us', 'en_US.',
                          'en_us.utf8',
                          'en_us.UTF-8',
@@ -76,12 +78,16 @@ class CommonTestCase(common.BleachbitTestCase):
         from bleachbit import locale_dir
         locale_dirs = list(set([locale_dir, '/usr/share/locale']))
         lang_codes = []
+        # Skip directories that are not valid language codes
+        skip_dirs = {'l10n'}
         for locale_dir in locale_dirs:
             if not os.path.isdir(locale_dir):
                 continue
             for lang_code in os.listdir(locale_dir):
                 if not os.path.isdir(os.path.join(locale_dir, lang_code)):
                     continue
+                if lang_code in skip_dirs:
+                    continue
                 lang_codes.append(lang_code)
         if os.path.exists('/etc/locale.alias'):
             with open('/etc/locale.alias', 'r') as f:
@@ -91,7 +97,13 @@ class CommonTestCase(common.BleachbitTestCase):
                         parts = line.split()
                         if len(parts) > 1:
                             lang_codes.append(parts[1])
+        # /etc/locale.alias may list the qaa-qtz range, which is reserved for
+        # private use rather than a concrete locale. Skip it if present.
+        skip_alias_codes = {'qaa-qtz', 'it_CARES'}
+
         for lang_code in lang_codes:
+            if lang_code in skip_alias_codes:
+                continue
             self.assertIsLanguageCode(lang_code)
 
     def test_environment(self):
@@ -99,12 +111,13 @@ class CommonTestCase(common.BleachbitTestCase):
         # useful for researching
         # grep -Poh "([\\$%]\w+)" cleaners/*xml | cut -b2- | sort | uniq -i
         # bleachbit/init.py sets these variables on posix, if they do not exist.
-        envs = {'posix': ['HOME', 'USER', 'XDG_CACHE_HOME', 'XDG_CONFIG_HOME', 'XDG_DATA_HOME'],
+        # Docker does not set USER
+        envs = {'posix': ['HOME', 'XDG_CACHE_HOME', 'XDG_CONFIG_HOME', 'XDG_DATA_HOME'],
                 'nt': ['AppData', 'CommonAppData', 'Documents', 'ProgramFiles', 'UserProfile', 'WinDir']}
         for env in envs[os.name]:
             e = os.getenv(env)
-            self.assertIsNotNone(e)
-            self.assertNotEqual(e.strip(), '')
+            self.assertIsNotNone(e, f"Environment variable {env} is not set")
+            self.assertNotEqual(e.strip(), "", f"Environment variable {env} is empty")
 
     def test_get_put_env(self):
         """Unit test for get_env() and put_env()"""
@@ -116,6 +129,22 @@ class CommonTestCase(common.BleachbitTestCase):
         common.put_env('PUTENV_TEST', None)
         self.assertIsNone(common.get_env('PUTENV_TEST'))
 
+    @common.skipUnlessWindows
+    def test_get_opened_windows_titles(self):
+        """Unit test for get_opened_windows_titles()"""
+        titles = common.get_opened_windows_titles()
+
+        self.assertIsInstance(titles, list, "Should return a list")
+        self.assertGreater(len(titles), 0,
+                           f"Should have at least one window title. Got: {titles}")
+
+        # Verify all items are strings and none are empty
+        for title in titles:
+            self.assertIsInstance(
+                title, str, f"All titles should be strings, got {type(title)}")
+            self.assertGreater(len(title.strip()), 0,
+                               f"Window title should not be empty or whitespace-only: '{title}'")
+
     def test_touch_file(self):
         """Unit test for touch_file"""
         fn = os.path.join(self.tempdir, 'test_touch_file')
