@@ -31,6 +31,7 @@ import sys
 import tempfile
 import time
 import warnings
+from unittest import mock
 
 # local
 from bleachbit import logger
@@ -45,7 +46,7 @@ from bleachbit.General import (
     shell_split,
     sudo_mode)
 from tests import common
-from tests.common import test_also_with_sudo
+from tests.common import also_with_sudo
 
 
 class GeneralTestCase(common.BleachbitTestCase):
@@ -71,7 +72,7 @@ class GeneralTestCase(common.BleachbitTestCase):
         if sys.executable:
             self.assertEqual(exe, sys.executable)
 
-    @test_also_with_sudo
+    @also_with_sudo
     def test_get_real_uid(self):
         """Test for get_real_uid()"""
         if 'posix' != os.name:
@@ -143,7 +144,20 @@ class GeneralTestCase(common.BleachbitTestCase):
         uid_after = get_real_uid()
         self.assertEqual(uid, uid_after)
 
-    @test_also_with_sudo
+    def test_get_real_uid_numeric_login_without_passwd_entry(self):
+        """Docker containers may advertise UID as username without /etc/passwd."""
+        if 'posix' != os.name:
+            self.skipTest('POSIX-only behavior')
+
+        # Ensure the code path executes past the SUDO_UID shortcut.
+        env_overrides = {'SUDO_UID': ''}
+        with mock.patch.dict(os.environ, env_overrides, clear=False):
+            with mock.patch('os.getlogin', return_value='1000'):
+                with mock.patch('pwd.getpwnam', side_effect=KeyError):
+                    uid = get_real_uid()
+        self.assertEqual(uid, 1000)
+
+    @also_with_sudo
     def test_get_real_username(self):
         """Test for get_real_username()"""
         if 'posix' != os.name:
@@ -155,7 +169,32 @@ class GeneralTestCase(common.BleachbitTestCase):
         if sudo_mode():
             self.assertNotEqual(username, 'root')
 
-    @test_also_with_sudo
+    def test_get_real_username_container_env_fallback(self):
+        """Ensure container fallback uses LOGNAME when getpass fails."""
+        if 'posix' != os.name:
+            self.skipTest('POSIX-only behavior')
+
+        env = {'LOGNAME': 'containeruser'}
+        with mock.patch.dict(os.environ, env, clear=True):
+            with mock.patch('os.getlogin', side_effect=OSError('no tty')):
+                with mock.patch('getpass.getuser', side_effect=KeyError):
+                    username = get_real_username()
+        self.assertEqual(username, 'containeruser')
+
+    def test_get_real_username_container_uid_fallback(self):
+        """Ensure fallback returns UID string when everything else fails."""
+        if 'posix' != os.name:
+            self.skipTest('POSIX-only behavior')
+
+        env = {}
+        with mock.patch.dict(os.environ, env, clear=True):
+            with mock.patch('os.getlogin', side_effect=OSError('no tty')):
+                with mock.patch('getpass.getuser', side_effect=OSError('no user')):
+                    with mock.patch('os.getuid', return_value=4242):
+                        username = get_real_username()
+        self.assertEqual(username, '4242')
+
+    @also_with_sudo
     def test_makedirs(self):
         """Unit test for makedirs"""
 
@@ -236,11 +275,17 @@ class GeneralTestCase(common.BleachbitTestCase):
                 self.assertNotExists(test_file)
 
                 # Test with wait=False
-                (rc, stdout, stderr) = run_external(cmd, wait=False)
+                (rc, stdout, stderr) = run_external(cmd, wait=False, timeout=5)
+
                 self.assertEqual(0, rc)
                 self.assertEqual('', stdout)
                 self.assertEqual('', stderr)
-                time.sleep(1)
+                # Poll for file creation. Touch normally completes in milliseconds,
+                # so one second is plenty.
+                for _ in range(100):
+                    if os.path.exists(test_file):
+                        break
+                    time.sleep(0.01)
                 self.assertExists(test_file)
 
     def test_run_external_with_timeout_failure(self):
@@ -376,8 +421,8 @@ class GeneralTestCase(common.BleachbitTestCase):
         if sudo_mode():
             self.skipTest('dconf not supported in sudo mode')
         # pylint: disable=import-outside-toplevel
-        from bleachbit.Unix import has_gui
-        if not has_gui():
+        from bleachbit.GtkShim import HAVE_GTK
+        if not HAVE_GTK:
             self.skipTest('dconf not supported without GUI')
         args = ['dconf', 'write',
                 '/apps/bleachbit/test', 'true']
@@ -405,7 +450,7 @@ class GeneralTestCase(common.BleachbitTestCase):
             self.assertEqual(test[1], shell_split(test[0]))
 
     @common.skipIfWindows
-    @test_also_with_sudo
+    @also_with_sudo
     def test_sudo_mode(self):
         """Unit test for sudo_mode"""
         self.assertIsInstance(sudo_mode(), bool)
