@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,11 +22,13 @@
 Test case for module Options
 """
 
+import errno
+import os
+from unittest import mock
+
 from tests import common
 import bleachbit.Options
-from bleachbit import NoOptionError
-
-import os
+from bleachbit.Log import is_debugging_enabled_via_cli
 
 
 class OptionsTestCase(common.BleachbitTestCase):
@@ -159,7 +161,7 @@ class OptionsTestCase(common.BleachbitTestCase):
         # write something, which triggers the purge
         o3.set('dummypath', 'dummyvalue', 'hashpath')
         # verify the path was purged
-        self.assertRaises(NoOptionError, lambda: o3.get_hashpath(pathname))
+        self.assertIsNone(o3.get_hashpath(pathname))
 
     def test_abbreviations(self):
         """Test non-standard, abbreviated booleans T and F"""
@@ -196,3 +198,117 @@ class OptionsTestCase(common.BleachbitTestCase):
 
         # clean up
         del opt
+
+    def test_error_disk_full(self):
+        """Test graceful degradation when disk is full"""
+        disk_full_error = OSError('No space left on device')
+        disk_full_error.errno = errno.ENOSPC
+        o = bleachbit.Options.Options()
+        with mock.patch('builtins.open', side_effect=disk_full_error):
+            with self.assertLogs(level='ERROR') as log_context:
+                o.set('test_key', 'test_value')
+        self.assertIn('Disk was full', log_context.output[0])
+        self.assertIn(bleachbit.options_file, log_context.output[0])
+        self.assertEqual(o.get('test_key'), 'test_value')
+
+    def test_error_permission(self):
+        """Test graceful degradation with permission errors"""
+        permission_error = PermissionError('Permission denied')
+        permission_error.errno = errno.EACCES
+        o = bleachbit.Options.Options()
+        with mock.patch('builtins.open', side_effect=permission_error):
+            with self.assertLogs(level='ERROR') as log_context:
+                o.set('test_key', 'test_value')
+        self.assertIn('Permission denied', log_context.output[0])
+        self.assertIn(bleachbit.options_file, log_context.output[0])
+        self.assertEqual(o.get('test_key'), 'test_value')
+
+    def test_warning(self):
+        """Test warning preferences"""
+        o = bleachbit.Options.options
+        self.assertFalse(o.get_warning_preference('test_key'))
+        o.remember_warning_preference('test_key')
+        self.assertTrue(o.get_warning_preference('test_key'))
+
+    def test_overrides(self):
+        """Test CLI override functionality"""
+        o = bleachbit.Options.options
+
+        debug_default = is_debugging_enabled_via_cli()
+
+        # Check defaults which are assumed in later tests.
+        self.assertFalse(o.get('shred'))
+        self.assertEqual(o.get('debug'), debug_default)
+        self.assertTrue(o.get('delete_confirmation'))
+
+        # Test basic override
+        o.set_override('shred', True)
+        self.assertTrue(o.get('shred'))
+
+        # Test that override takes precedence over get
+        o.set('shred', False, commit=False)
+        self.assertTrue(o.get('shred'))  # override should still return True
+
+        # Test multiple overrides
+        o.set_override('debug', True)
+        o.set_override('delete_confirmation', False)
+        self.assertTrue(o.get('debug'))
+        self.assertFalse(o.get('delete_confirmation'))
+
+        # Test that flush doesn't write overridden values
+        # First set persistent values different from overrides
+        o.set('shred', False, commit=False)
+        o.set('debug', False, commit=False)
+        o.set('delete_confirmation', True, commit=False)
+        o.commit()
+
+        # Verify overrides still work after flush
+        self.assertTrue(o.get('shred'))
+        self.assertTrue(o.get('debug'))
+        self.assertFalse(o.get('delete_confirmation'))
+
+        # Create new instance to verify persistent values were written correctly
+        o2 = bleachbit.Options.Options()
+        self.assertFalse(o2.get('shred'))
+        self.assertEqual(o2.get('debug'), debug_default)
+        self.assertTrue(o2.get('delete_confirmation'))
+
+    def test_override_does_not_persist(self):
+        """Test that overrides don't persist after process restart"""
+        o = bleachbit.Options.options
+
+        # Set an override
+        o.set_override('shred', True)
+        self.assertTrue(o.get('shred'))
+
+        # Create new instance (simulating process restart)
+        o2 = bleachbit.Options.Options()
+        # Override should not exist in new instance
+        self.assertNotIn(('bleachbit', 'shred'), o2.overrides)
+
+        # Clean up
+        o.reset_overrides()
+
+    def test_override_with_sections(self):
+        """Test overrides work with different sections"""
+        o = bleachbit.Options.options
+
+        # Test override in non-default section
+        if not o.config.has_section('test_section'):
+            o.config.add_section('test_section')
+        o.config.set('test_section', 'test_key', 'original_value')
+
+        o.set_override('test_key', 'override_value', section='test_section')
+        self.assertEqual(
+            o.get('test_key', section='test_section'), 'override_value')
+
+        # Verify flush doesn't write override
+        o.commit()
+        o2 = bleachbit.Options.Options()
+        self.assertEqual(
+            o2.get('test_key', section='test_section'), 'original_value')
+
+        # Clean up
+        o.reset_overrides()
+        if o.config.has_section('test_section'):
+            o.config.remove_section('test_section')
