@@ -35,7 +35,7 @@ from xml.dom.minidom import parseString
 from tests import common
 from bleachbit import logger
 from bleachbit.General import get_executable, get_real_username
-from tests.common import test_also_with_sudo
+from tests.common import also_with_sudo
 from bleachbit.FileUtilities import children_in_directory, exe_exists
 from bleachbit.Unix import (
     _is_broken_xdg_desktop_application,
@@ -51,7 +51,6 @@ from bleachbit.Unix import (
     get_distribution_name_version_platform_freedesktop,
     get_distribution_name_version,
     get_purgeable_locales,
-    has_gui,
     is_broken_xdg_desktop,
     is_process_running_ps_aux,
     is_process_running,
@@ -420,7 +419,7 @@ PrefersNonDefaultGPU=false""")
 
     @mock.patch('subprocess.check_output')
     @common.skipIfWindows
-    @test_also_with_sudo
+    @also_with_sudo
     def test_is_process_running_ps_aux(self, mock_check_output):
         username = get_real_username()
         ps_out = f"""USER               PID  %CPU %MEM      VSZ    RSS   TT  STAT STARTED      TIME COMMAND
@@ -458,7 +457,7 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
         self.assertRaises(RuntimeError, is_process_running_ps_aux, 'foo', True)
 
     @common.skipIfWindows
-    @common.test_also_with_sudo
+    @common.also_with_sudo
     def test_is_process_running(self):
         """Unit test for method is_process_running()"""
         # Do not use get_executable() here because of how it
@@ -777,6 +776,25 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
             self.assertIsInteger(bytes_freed)
             logger.debug('pacman bytes cleaned %d', bytes_freed)
 
+    @common.skipIfWindows
+    @mock.patch('bleachbit.Unix.General.run_external')
+    @mock.patch('bleachbit.Unix.exe_exists')
+    @mock.patch('bleachbit.Unix.os.path')
+    def test_pacman_cache_mock(self, mock_path, mock_exe_exists, mock_run):
+        """Unit test pacman_cache() parse logic without pacman installed"""
+        mock_path.exists.return_value = False
+        mock_exe_exists.return_value = True
+        mock_run.return_value = (
+            0,
+            "==> finished: 3 packages removed (42.31 M freed)\n",
+            ''
+        )
+
+        bytes_freed = pacman_cache()
+        self.assertEqual(bytes_freed, 42310000)
+
+        mock_run.assert_called_once_with(['paccache', '-rk0'])
+
     @common.skipIfWindows
     @common.skipUnlessDestructive
     def test_snap_disabled_clean(self):
@@ -821,11 +839,6 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
         result = snap_parse_list(sample)
         self.assertEqual(result, [])
 
-    @common.skipIfWindows
-    def test_has_gui(self):
-        """Unit test for has_gui()"""
-        self.assertIsInstance(has_gui(), bool)
-
     @common.skipIfWindows
     def test_is_unix_display_protocol_wayland_no_mock(self):
         """Unit test for test_is_unix_display_protocol_wayland() with no mock"""
