@@ -23,6 +23,7 @@
 Test case for module Windows
 """
 
+
 from tests import common
 
 from bleachbit import FileUtilities, General
@@ -51,24 +52,38 @@ from bleachbit.Windows import (
     set_environ,
     setup_environment,
     shell_change_notify,
-    split_registry_key
+    split_registry_key,
+    read_registry_key,
+    get_sid_token_48,
+    is_ots_elevation,
+    SplashThread,
 )
 from bleachbit import logger
 
+import ctypes
 import os
+import itertools
 import platform
 import shutil
+import subprocess
 import sys
+import time
 import tempfile
 from decimal import Decimal
-import time
+from pathlib import Path
+from unittest import mock
+from random import randint
+
 
 if 'win32' == sys.platform:
     import pywintypes
     import win32api
+    import win32service
     import winreg
     from win32com.shell import shell
 
+    from bleachbit import Windows
+
 
 def put_files_into_recycle_bin():
     """Put a file and a folder into the recycle bin"""
@@ -85,8 +100,99 @@ def put_files_into_recycle_bin():
     move_to_recycle_bin(dirname)
 
 
+class WindowsLinksMixIn():
+    """Mixin class for Windows link creation methods."""
+
+    def _create_win_dir_symlink(self, target, linkname):
+        """Create a directory symlink"""
+
+        self.assertFalse(os.path.lexists(linkname),
+                         f'Link must not exist: {linkname}')
+        self.assertTrue(os.path.isabs(target),
+                        f'Target must be absolute path: {target}')
+        self.assertTrue(os.path.isabs(linkname),
+                        f'Link must be absolute path: {linkname}')
+        self.assertExists(target)
+        target_path = Path(target)
+        self.assertTrue(target_path.is_dir(),
+                        f'Target must be an existing directory: {target}')
+
+        kernel32 = ctypes.windll.kernel32
+        kernel32.CreateSymbolicLinkW.argtypes = [
+            ctypes.c_wchar_p,
+            ctypes.c_wchar_p,
+            ctypes.c_uint32,
+        ]
+        kernel32.CreateSymbolicLinkW.restype = ctypes.c_ubyte
+        result = kernel32.CreateSymbolicLinkW(
+            linkname, target, 1)  # SYMBOLIC_LINK_FLAG_DIRECTORY
+        if result == 0:
+            err = ctypes.GetLastError()
+            raise OSError(err, ctypes.FormatError(err))
+        self.assertExists(linkname)
+        link_path = Path(linkname)
+        self.assertTrue(link_path.is_symlink())
+        self.assertTrue(link_path.is_dir())
+        self.assertFalse(Windows.is_junction(linkname))
+        self.assertFalse(FileUtilities.is_normal_directory(linkname))
+
+    def _create_win_hard_link(self, target, linkname):
+        """Create a hard link to a file"""
+
+        self.assertFalse(os.path.lexists(linkname),
+                         f'Link must not exist: {linkname}')
+        self.assertTrue(os.path.isabs(target),
+                        f'Target must be absolute path: {target}')
+        self.assertTrue(os.path.isabs(linkname),
+                        f'Link must be absolute path: {linkname}')
+        self.assertExists(target)
+        target_path = Path(target)
+        self.assertTrue(target_path.is_file(),
+                        f'Target must be an existing file: {target}')
+        kernel32 = ctypes.windll.kernel32
+        kernel32.CreateHardLinkW.argtypes = [
+            ctypes.c_wchar_p,
+            ctypes.c_wchar_p,
+            ctypes.c_void_p,
+        ]
+        kernel32.CreateHardLinkW.restype = ctypes.c_bool
+        result = kernel32.CreateHardLinkW(linkname, target, None)
+        if not result:
+            err = ctypes.GetLastError()
+            raise OSError(err, ctypes.FormatError(err))
+        self.assertExists(linkname)
+        link_path = Path(linkname)
+        self.assertTrue(link_path.is_file())
+        self.assertFalse(link_path.is_symlink())
+        self.assertFalse(Windows.is_junction(linkname))
+        self.assertTrue(os.path.samefile(target, linkname))
+        self.assertFalse(FileUtilities.is_normal_directory(linkname))
+
+    def _create_win_junction(self, target, linkname):
+        """Create a directory junction using mklink /J."""
+
+        if os.path.lexists(linkname):
+            raise OSError(f'Link already exists: {linkname}')
+        self.assertTrue(os.path.isabs(target),
+                        f'Target must be absolute path: {target}')
+        self.assertTrue(os.path.isabs(linkname),
+                        f'Link must be absolute path: {linkname}')
+        self.assertExists(target)
+        target_path = Path(target)
+        self.assertTrue(target_path.is_dir(),
+                        f'Target must be an existing directory: {target}')
+        cmd = ['cmd', '/c', 'mklink', '/J',
+               extended_path(linkname), extended_path(target)]
+        subprocess.check_call(cmd)
+        self.assertExists(linkname)
+        self.assertTrue(Windows.is_junction(linkname))
+        path = Path(linkname)
+        self.assertTrue(path.is_dir())
+        self.assertFalse(path.is_symlink())
+
+
 @common.skipUnlessWindows
-class WindowsTestCase(common.BleachbitTestCase):
+class WindowsTestCase(common.BleachbitTestCase, WindowsLinksMixIn):
 
     """Test case for module Windows"""
 
@@ -113,54 +219,48 @@ class WindowsTestCase(common.BleachbitTestCase):
         for _f in get_recycle_bin():
             self.fail('recycle bin should be empty, but it is not')
 
-    def _test_link_helper(self, mklink_option, clear_recycle_bin):
-        """Helper function for testing for links with is_junction() and
-        get_recycle_bin()
+    def _test_link_helper(self, mklink_option, recycle_container, clear_recycle_bin):
+        """Helper function for testing directory junctions and symlinks in the recycle bin.
 
-        It gets called four times for the combinations of the two
-        parameters. It's called by four unit tests for accounting
-        purposes. In other words, we don't want to count a test as
-        skipped if part of it succeeded.
+        Tests that get_recycle_bin() correctly handles Windows directory links
+        when they are placed in the recycle bin, and verifies that the original
+        target directory and its contents are preserved after clearing the bin.
 
-        mklink /j = directory junction
-        directory junction does not require administrator privileges
+        Args:
+            mklink_option: Link type to create. '/j' for directory junction,
+                '/d' for directory symbolic link.
+            recycle_container: If True, move the container directory to the
+                recycle bin. If False, move only the link itself.
+            clear_recycle_bin: If True, clear the recycle bin after moving
+                the link. If False, manually delete items from get_recycle_bin().
 
-        mklink /d=directory symbolic link
-        requires administrator privileges
+        Note:
+            Directory symbolic links (/d) require administrator privileges.
         """
+        assert mklink_option in ('/j', '/d')
         if mklink_option == '/d':
             self.skipUnlessAdmin()
         # make a normal directory with a file in it
-        target_dir = os.path.join(self.tempdir, 'target_dir')
-        os.mkdir(target_dir)
-        self.assertExists(target_dir)
-        self.assertFalse(is_junction(target_dir))
+        target_dir = self.mkdir('target_dir')
 
-        from random import randint
-        canary_fn = os.path.join(
-            target_dir, 'do_not_delete%d' % randint(1000, 9999))
+        canary_base = f'do_not_delete{randint(10000, 9999999)}'
+        canary_fn = os.path.join(target_dir, canary_base)
         common.touch_file(canary_fn)
-        self.assertExists(canary_fn)
         self.assertFalse(is_junction(canary_fn))
 
         # make a normal directory to hold a link
-        container_dir = os.path.join(self.tempdir, 'container_dir')
-        os.mkdir(container_dir)
-        self.assertExists(container_dir)
-        self.assertFalse(is_junction(container_dir))
+        container_dir = self.mkdir('container_dir')
 
         # create the link
         link_pathname = os.path.join(container_dir, 'link')
-        args = ('cmd', '/c', 'mklink', mklink_option,
-                link_pathname, target_dir)
-        from bleachbit.General import run_external
-        (rc, stdout, stderr) = run_external(args)
-        self.assertEqual(rc, 0, stderr)
-        self.assertExists(link_pathname)
-        self.assertTrue(is_junction(link_pathname))
+        if mklink_option == '/j':
+            self._create_win_junction(target_dir, link_pathname)
+        else:
+            self._create_win_dir_symlink(target_dir, link_pathname)
 
         # put the link in the recycle bin
-        move_to_recycle_bin(container_dir)
+        move_to_recycle_bin(
+            container_dir if recycle_container else link_pathname)
 
         def cleanup_dirs():
             shutil.rmtree(container_dir, True)
@@ -173,6 +273,8 @@ class WindowsTestCase(common.BleachbitTestCase):
 
         # clear the recycle bin
         for f in get_recycle_bin():
+            if canary_base in f:
+                logger.error('get_recycle_bin() returned canary: %s', f)
             FileUtilities.delete(f, shred=False)
 
         # verify the canary is still there
@@ -181,21 +283,16 @@ class WindowsTestCase(common.BleachbitTestCase):
         # clean up
         cleanup_dirs()
 
-    def test_link_junction_no_clear(self):
-        """Unit test for directory junctions without clearing recycle bin"""
-        self._test_link_helper('/j', False)
-
-    def test_link_junction_clear(self):
-        """Unit test for directory junctions with clearing recycle bin"""
-        self._test_link_helper('/j', True)
-
-    def test_link_symlink_no_clear(self):
-        """Unit test for directory symlink without clearing recycle bin"""
-        self._test_link_helper('/d', False)
-
-    def test_link_symlink_clear(self):
-        """Unit test for directory symlink with clearing recycle bin"""
-        self._test_link_helper('/d', True)
+    def test_link_types(self):
+        """Unit test for directory junctions and symlinks with recycle bin"""
+        for mklink_option, recycle_container, clear_recycle_bin in itertools.product(
+            ('/j', '/d'), (False, True), (False, True)
+        ):
+            with self.subTest(mklink_option=mklink_option,
+                              recycle_container=recycle_container,
+                              clear_recycle_bin=clear_recycle_bin):
+                self._test_link_helper(
+                    mklink_option, recycle_container, clear_recycle_bin)
 
     def test_delete_locked_file(self):
         """Unit test for delete_locked_file"""
@@ -289,6 +386,86 @@ class WindowsTestCase(common.BleachbitTestCase):
         with self.assertRaises(AssertionError):
             is_service_running(None)
 
+    def test_is_ots_elevation_without_flag_returns_false(self):
+        """Without --uac-sid-token, is_ots_elevation() returns False"""
+        argv = ['bleachbit.exe', '--gui']
+        with mock.patch('bleachbit.Windows.sys.argv', argv):
+            self.assertFalse(is_ots_elevation())
+
+    def test_is_ots_elevation_false_when_tokens_match(self):
+        """If current token matches parent token, there is no elevation"""
+        parent_token = 'ABCDEFGH'
+        argv = ['bleachbit.exe', '--gui', '--uac-sid-token', parent_token]
+        with mock.patch('bleachbit.Windows.sys.argv', argv):
+            with mock.patch('bleachbit.Windows.get_sid_token_48', return_value=parent_token):
+                self.assertFalse(is_ots_elevation())
+
+    def test_is_ots_elevation_true_when_tokens_differ(self):
+        """If current token differs from parent token, elevation is detected"""
+        parent_token = 'ABCDEFGH'
+        argv = ['bleachbit.exe', '--gui', '--uac-sid-token', parent_token]
+        with mock.patch('bleachbit.Windows.sys.argv', argv):
+            with mock.patch('bleachbit.Windows.get_sid_token_48', return_value='DIFFERNT'):
+                self.assertTrue(is_ots_elevation())
+
+    def test_is_ots_elevation_ignores_flag_without_value(self):
+        """A trailing --uac-sid-token without value should not crash and returns False"""
+        argv = ['bleachbit.exe', '--gui', '--uac-sid-token']
+        with mock.patch('bleachbit.Windows.sys.argv', argv):
+            self.assertFalse(is_ots_elevation())
+
+    def test_is_ots_elevation_returns_false_on_get_sid_error(self):
+        """If get_sid_token_48() raises, is_ots_elevation() falls back to False"""
+        parent_token = 'ABCDEFGH'
+        argv = ['bleachbit.exe', '--gui', '--uac-sid-token', parent_token]
+        with mock.patch('bleachbit.Windows.sys.argv', argv):
+            with mock.patch('bleachbit.Windows.get_sid_token_48', side_effect=RuntimeError('error')):
+                self.assertFalse(is_ots_elevation())
+
+    def test_splash_thread_reuses_cached_class_atom(self):
+        """_register_window_class skips RegisterClass when cached."""
+        splash = SplashThread()
+        self.addCleanup(lambda: setattr(SplashThread, '_class_atom', None))
+        SplashThread._class_atom = 9876
+        with mock.patch('bleachbit.Windows.win32gui.RegisterClass') as mock_register:
+            atom = splash._register_window_class(mock.Mock())
+        self.assertEqual(atom, 9876)
+        mock_register.assert_not_called()
+
+    def test_splash_thread_recovers_when_class_exists(self):
+        """_register_window_class handles ERROR_CLASS_ALREADY_EXISTS."""
+        splash = SplashThread()
+        self.addCleanup(lambda: setattr(SplashThread, '_class_atom', None))
+        SplashThread._class_atom = None
+        wnd_class = mock.Mock()
+        wnd_class.hInstance = mock.sentinel.instance
+        wnd_class.lpszClassName = 'SimpleWin32'
+        register_error = pywintypes.error(1410, 'RegisterClass', 'Class already exists.')
+        with mock.patch('bleachbit.Windows.win32gui.RegisterClass', side_effect=register_error) as mock_register, \
+                mock.patch('bleachbit.Windows.win32gui.GetClassInfo', return_value=(4321,)) as mock_get_class_info:
+            atom = splash._register_window_class(wnd_class)
+        self.assertEqual(atom, 4321)
+        self.assertEqual(SplashThread._class_atom, 4321)
+        mock_register.assert_called_once_with(wnd_class)
+        mock_get_class_info.assert_called_once_with(wnd_class.hInstance, wnd_class.lpszClassName)
+
+    def test_splash_thread_propagates_startup_error(self):
+        """start() raises if splash initialization fails."""
+        splash = SplashThread()
+        with mock.patch.object(splash, '_show_splash_screen', side_effect=RuntimeError('boom')):
+            with self.assertRaises(RuntimeError):
+                splash.start()
+        self.assertIsNotNone(splash._startup_error)
+
+    def test_splash_thread_join_handles_missing_window(self):
+        """join() succeeds even when window handle was never created."""
+        splash = SplashThread()
+        splash._startup_error = RuntimeError('boom')
+        splash._splash_screen_started.set()
+        splash.start = lambda: None  # prevent thread start
+        # Directly call join; should not raise even though handle is None
+        splash.join(timeout=0)
+
     @common.skipUnlessDestructive
     def test_run_net_service_command(self):
         """Integration test for run_net_service_command().
@@ -304,30 +481,83 @@ class WindowsTestCase(common.BleachbitTestCase):
         """
         if not shell.IsUserAnAdmin():
             self.skipTest('requires administrator privileges')
-        for service in ('AudioEndpointBuilder', 'spooler'):
-
-            initial_running = is_service_running(service)
-
+        def _service_exists_and_enabled(svc):
+            scm = win32service.OpenSCManager(None, None, win32service.SC_MANAGER_CONNECT)
             try:
-                # Stop service
-                run_net_service_command(service, False)
+                hs = win32service.OpenService(scm, svc, win32service.SERVICE_QUERY_STATUS | win32service.SERVICE_QUERY_CONFIG)
+                try:
+                    cfg = win32service.QueryServiceConfig(hs)
+                    return True if cfg[1] != win32service.SERVICE_DISABLED else False
+                finally:
+                    win32service.CloseServiceHandle(hs)
+            except pywintypes.error:
+                return False
+            finally:
+                win32service.CloseServiceHandle(scm)
+
+        def _can_open_all_access(svc):
+            scm = win32service.OpenSCManager(None, None, win32service.SC_MANAGER_CONNECT)
+            try:
+                try:
+                    hs = win32service.OpenService(scm, svc, win32service.SERVICE_ALL_ACCESS)
+                except pywintypes.error:
+                    return False
+                else:
+                    win32service.CloseServiceHandle(hs)
+                    return True
+            finally:
+                win32service.CloseServiceHandle(scm)
+
+        is_ci = os.environ.get('APPVEYOR') == 'True'
+        if is_ci:
+            candidates = ('bits', 'wuauserv', 'AudioEndpointBuilder', 'spooler')
+            service = None
+            # Prefer already-running to avoid state changes
+            for s in candidates:
+                if _service_exists_and_enabled(s) and _can_open_all_access(s) and is_service_running(s):
+                    service = s
+                    break
+            # If none are running, pick any we can open with required access
+            if not service:
+                for s in candidates:
+                    if _service_exists_and_enabled(s) and _can_open_all_access(s):
+                        service = s
+                        break
+        else:
+            candidates = ('AudioEndpointBuilder', 'spooler', 'bits', 'wuauserv')
+            service = None
+            for s in candidates:
+                if _service_exists_and_enabled(s):
+                    service = s
+                    break
+        if not service:
+            self.skipTest('no suitable startable service on this machine')
+
+        initial_running = is_service_running(service)
+
+        try:
+            if is_ci:
+                ret = run_net_service_command(service, True)
+                self.assertEqual(ret, 0)
+                ret = run_net_service_command(service, True)
+                self.assertEqual(ret, 0)
+            else:
+                ret = run_net_service_command(service, False)
+                self.assertEqual(ret, 0)
                 self.assertFalse(is_service_running(service))
                 if service == 'AudioEndpointBuilder':
                     self.assertFalse(is_service_running('audiosrv'))
-
-                # Stop again
-                run_net_service_command(service, False)
+                ret = run_net_service_command(service, False)
+                self.assertEqual(ret, 0)
                 self.assertFalse(is_service_running(service))
-
-                # Start service
-                run_net_service_command(service, True)
+                ret = run_net_service_command(service, True)
+                self.assertEqual(ret, 0)
                 self.assertTrue(is_service_running(service))
-
-                # Start again
-                run_net_service_command(service, True)
+                ret = run_net_service_command(service, True)
+                self.assertEqual(ret, 0)
                 self.assertTrue(is_service_running(service))
-            finally:
-                # Restore initial state
+        finally:
+            if not is_ci:
                 run_net_service_command(service, initial_running)
                 self.assertEqual(initial_running, is_service_running(service))
 
@@ -451,6 +681,25 @@ class WindowsTestCase(common.BleachbitTestCase):
             self.assertEqual(drive, drive.upper())
         self.assertIn("C:\\", drives)
 
+    def test_get_sid_token_48_basic_properties(self):
+        """get_sid_token_48() returns an 8-char, URL-safe ASCII string"""
+        token = get_sid_token_48()
+        self.assertIsString(token)
+        # 6 bytes (48 bits) become 8 base64-url characters without padding
+        self.assertEqual(len(token), 8)
+        for ch in token:
+            self.assertLess(ord(ch), 128)
+        # urlsafe_b64encode must not use '+', '/', or '='
+        self.assertNotIn('+', token)
+        self.assertNotIn('/', token)
+        self.assertNotIn('=', token)
+
+    def test_get_sid_token_48_is_deterministic_for_current_process(self):
+        """Multiple calls for the same process should yield the same token"""
+        t1 = get_sid_token_48()
+        t2 = get_sid_token_48()
+        self.assertEqual(t1, t2)
+
     def test_get_windows_version(self):
         """Unit test for get_windows_version"""
         v = get_windows_version()
@@ -604,6 +853,16 @@ class WindowsTestCase(common.BleachbitTestCase):
             self.assertEqual(expected_hive, hive)
             self.assertEqual(expected_key, key)
 
+    def test_read_registry_key(self):
+        """Unit test for read_registry_key"""
+        tests = (('HKCR\\.bmp', 'PerceivedType', 'image'),
+                 ('HKCU\\Software\\BleachBit\\DoesNotExist', 'DoesNotExist', None))
+        for (input_key, input_value, expected_value) in tests:
+            value = read_registry_key(input_key, input_value)
+            if value != None:
+                value = value.lower() # AppVeyor: image, Windows 11: Image
+            self.assertEqual(expected_value, value)
+
     def test_parse_windows_build(self):
         """Unit test for parse_windows_build"""
         tests = (('5.1.2600', Decimal('5.1')),
@@ -629,6 +888,63 @@ class WindowsTestCase(common.BleachbitTestCase):
         ret = shell_change_notify()
         self.assertEqual(ret, 0)
 
+    def test_splash_screen(self):
+        """Unit test for splash screen"""
+        splash_thread = SplashThread()
+        icon_path = splash_thread.get_icon_path()
+        self.assertTrue(icon_path.is_absolute())
+        self.assertExists(icon_path)
+        splash_thread.start()
+        timeout = 5.0  # seconds
+        start_time = time.time()
+        time.sleep(1)
+        while not splash_thread.is_alive() and (time.time() - start_time) < timeout:
+            time.sleep(0.1)
+        self.assertTrue(splash_thread.is_alive(),
+                        'Splash thread did not start within timeout')
+        splash_thread.join()
+
+        # Originally, running twice in same process caused indefinite hang,
+        # so run it twice.
+        repeat = SplashThread()
+        repeat.start()
+        repeat.join()
+
+    def test_splash_screen_window_pos(self):
+        """Unit test for calculate_window_position()"""
+        splash_thread = SplashThread()
+
+        # Test with a typical 1080p display
+        display_width = 1920
+        display_height = 1080
+        x, y, width, height = splash_thread.calculate_window_position(
+            display_width, display_height)
+
+        # The window should have positive dimensions
+        self.assertGreater(width, 0)
+        self.assertGreater(height, 0)
+
+        # The window position should be positive (on screen)
+        self.assertGreaterEqual(x, 0)
+        self.assertGreaterEqual(y, 0)
+
+        # The window should fit within the display
+        self.assertLessEqual(x + width, display_width)
+        self.assertLessEqual(y + height, display_height)
+
+        # The window should be roughly centered
+        # x should be approximately (display_width - width) / 2
+        expected_x = (display_width - width) // 2
+        self.assertEqual(x, expected_x)
+        # y should be approximately (display_height - height) / 2
+        expected_y = (display_height - height) // 2
+        self.assertEqual(y, expected_y)
+
+        # Width should be 1/4 of display width (480 for 1920)
+        self.assertEqual(width, display_width // 4)
+        # Height should be 100
+        self.assertEqual(height, 100)
+
     def test_set_environ(self):
         for folder in ['folderäö', 'folder']:
             test_dir = os.path.join(self.tempdir, folder)
